@@ -109,9 +109,9 @@ static void diff_save_ways(way_t *way, xmlNodePtr root_node) {
   }
 }
 
-void diff_save(project_t *project, osm_t *osm) {
+/* return true if no diff needs to be saved */
+gboolean diff_is_clean(osm_t *osm) {
   gboolean clean = TRUE;
-  char *diff_name = g_strdup_printf("%s/%s.diff", project->path, project->name);
 
   /* check if a diff is necessary */
   node_t *node = osm->node;
@@ -126,7 +126,13 @@ void diff_save(project_t *project, osm_t *osm) {
     way = way->next;
   }
 
-  if(clean) {
+  return clean;
+}
+
+void diff_save(project_t *project, osm_t *osm) {
+  char *diff_name = g_strdup_printf("%s/%s.diff", project->path, project->name);
+
+  if(diff_is_clean(osm)) {
     printf("data set is clean, removing diff if present\n");
     g_remove(diff_name);
     g_free(diff_name);
@@ -222,7 +228,6 @@ static tag_t *xml_scan_tags(xmlDoc *doc, xmlNodePtr node, osm_t *osm) {
   return first_tag;
 }
 
-
 void diff_restore_node(xmlDoc *doc, xmlNodePtr node_node, osm_t *osm) {
   printf("Restoring node\n");
 
@@ -326,31 +331,36 @@ void diff_restore_way(xmlDoc *doc, xmlNodePtr node_node, osm_t *osm) {
   way_t *way = NULL;
   switch(state) {
   case OSM_FLAG_NEW:
-    printf("  Untested NEW\n");
+    printf("  Restoring NEW way\n");
     
     way = g_new0(way_t, 1);
-    way->flags |= OSM_FLAG_NEW;
-    /* TODO: setup and append to chain */
-    
-    g_assert(0);
+    way->visible = TRUE;
+    way->flags = OSM_FLAG_NEW;
+    way->time = xml_get_prop_int(node_node, "time", 0);
+    if(!way->time) way->time = time(NULL);
+
+    /* attach to end of way list */
+    way_t **lway = &osm->way;
+    while(*lway) lway = &(*lway)->next;  
+    *lway = way;
     break;
 
   case OSM_FLAG_DELETED:
-    printf("  Untested DELETE\n");
+    printf("  Restoring DELETE flag\n");
     
     if((way = osm_get_way_by_id(osm, id)))
       way->flags |= OSM_FLAG_DELETED;
-    
-    g_assert(0);
+    else
+      printf("  WARNING: no way with that id found\n");
     break;
 
   case OSM_FLAG_DIRTY:
     printf("  Valid id (DIRTY)\n");
     
-    if((way = osm_get_way_by_id(osm, id))) {
+    if((way = osm_get_way_by_id(osm, id))) 
       way->flags |= OSM_FLAG_DIRTY;
-      printf("  found existing way\n");
-    }
+    else
+      printf("  WARNING: no way with that id found\n");
     break;
 
   default:
