@@ -36,20 +36,18 @@ view_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
   if(gtk_tree_model_get_iter(model, &iter, path)) {
     g_assert(gtk_tree_path_get_depth(path) == 1);
 
-    if(!path_currently_selected) {
-      tag_t *tag;
-      gtk_tree_model_get(model, &iter, TAG_COL_DATA, &tag, -1);
+    tag_t *tag;
+    gtk_tree_model_get(model, &iter, TAG_COL_DATA, &tag, -1);
 
-      if(context->but_remove && context->but_edit) {
-
-	/* you just cannot delete or edit the "created_by" tag */
-	if(strcasecmp(tag->key, "created_by") == 0) {
-	  gtk_widget_set_sensitive(context->but_remove, FALSE);
-	  gtk_widget_set_sensitive(context->but_edit, FALSE);
-	} else {
-	  gtk_widget_set_sensitive(context->but_remove, TRUE);
-	  gtk_widget_set_sensitive(context->but_edit, TRUE);
-	}
+    if(context->but_remove && context->but_edit) {
+      
+      /* you just cannot delete or edit the "created_by" tag */
+      if(strcasecmp(tag->key, "created_by") == 0) {
+	gtk_widget_set_sensitive(context->but_remove, FALSE);
+	gtk_widget_set_sensitive(context->but_edit, FALSE);
+      } else {
+	gtk_widget_set_sensitive(context->but_remove, TRUE);
+	gtk_widget_set_sensitive(context->but_edit, TRUE);
       }
     }
   }
