@@ -286,7 +286,8 @@ static void attach_right(GtkWidget *table, GtkWidget *widget, gint y) {
 		   GTK_EXPAND | GTK_FILL, 0,0,0);
 }
 
-static tag_t **store_value(presets_widget_t *widget, tag_t **ctag, char *value) {
+static tag_t **store_value(presets_widget_t *widget, tag_t **ctag, 
+			   char *value) {
   if((value && strlen(value)) || !widget->del_if_empty) {
     *ctag = g_new0(tag_t, 1);
     (*ctag)->key = g_strdup(widget->key);
@@ -307,7 +308,8 @@ static gint table_expose_event(GtkWidget *widget, GdkEventExpose *event,
 			 gboolean *first) {
 
   if(*first) {
-    guint border_width = gtk_container_get_border_width(GTK_CONTAINER(widget->parent));
+    guint border_width = 
+      gtk_container_get_border_width(GTK_CONTAINER(widget->parent));
     gtk_viewport_set_shadow_type(GTK_VIEWPORT(widget->parent), GTK_SHADOW_NONE);
 
     gtk_widget_set_size_request(GTK_WIDGET(widget->parent), -1, 
@@ -318,7 +320,8 @@ static gint table_expose_event(GtkWidget *widget, GdkEventExpose *event,
 }
 #endif
 
-static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item, tag_t *orig_tag) {
+static tag_t *presets_item_dialog(GtkWindow *parent, 
+		     presets_item_t *item, tag_t *orig_tag) {
   gboolean ok = FALSE;
   tag_t *tag = NULL, **ctag = &tag;
 
@@ -358,8 +361,10 @@ static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item, tag_t
       widget_skip++;   // this widget isn't part of the contents anymore
       widget = widget->next;
 
-      /* skip all following separators */
-      while(widget && (widget->type == WIDGET_TYPE_SEPARATOR)) {
+      /* skip all following separators (and keys) */
+      while(widget && 
+	    ((widget->type == WIDGET_TYPE_SEPARATOR) ||
+	     (widget->type == WIDGET_TYPE_KEY))) {
 	widget_skip++;   // this widget isn't part of the contents anymore
 	widget = widget->next;
       }
@@ -406,12 +411,15 @@ static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item, tag_t
 	
       case WIDGET_TYPE_CHECK:
 	{ gboolean def = FALSE;
-	if(preset) def = strcasecmp(preset, "true");
-	else       def = widget->check_w.def;
+	  if(preset) def = ((strcasecmp(preset, "true") == 0) ||
+			    (strcasecmp(preset, "yes") == 0));
+	  else       def = widget->check_w.def;
 
-	gtk_widgets[widget_cnt] = gtk_check_button_new_with_label(widget->text);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_widgets[widget_cnt]), def);
-	attach_right(table, gtk_widgets[widget_cnt], widget_cnt-widget_skip);
+	  gtk_widgets[widget_cnt] = 
+	    gtk_check_button_new_with_label(widget->text);
+	  gtk_toggle_button_set_active(
+	    GTK_TOGGLE_BUTTON(gtk_widgets[widget_cnt]), def);
+	  attach_right(table, gtk_widgets[widget_cnt], widget_cnt-widget_skip);
       } break;
       
     case WIDGET_TYPE_TEXT:
