@@ -141,6 +141,8 @@ cb_menu_redraw(GtkWidget *window, gpointer data) {
   appdata->track.track = track_restore(appdata, appdata->project);
   if(appdata->track.track)
     map_track_draw(appdata->map, appdata->track.track);
+
+  wms_load(appdata);
 }
 #endif
 
@@ -159,7 +161,7 @@ static void
 cb_menu_zoomin(GtkWidget *widget, appdata_t *appdata) {
   if(!appdata || !appdata->map) return;
 
-  map_set_zoom(appdata->map, appdata->map->zoom*2);
+  map_set_zoom(appdata->map, appdata->map->zoom*ZOOM_FACTOR_MENU);
   printf("zoom is now %f\n", appdata->map->zoom);
 }
 
@@ -167,7 +169,7 @@ static void
 cb_menu_zoomout(GtkWidget *widget, appdata_t *appdata) {
   if(!appdata || !appdata->map) return;
 
-  map_set_zoom(appdata->map, appdata->map->zoom/2);
+  map_set_zoom(appdata->map, appdata->map->zoom/ZOOM_FACTOR_MENU);
   printf("zoom is now %f\n", appdata->map->zoom);
 }
 #endif
@@ -388,10 +390,14 @@ void cleanup(appdata_t *appdata) {
   appdata->program = NULL;
 #endif
 
+  printf("waiting for gtk to shut down ...");
+
   /* let gtk clean up first */
   while(gtk_events_pending())
     gtk_main_iteration();
 
+  printf("ok\n");
+
   /* save a dirty project file */
   if(appdata->project && appdata->project->dirty)
     project_save(GTK_WIDGET(appdata->window), appdata->project);
