@@ -36,7 +36,7 @@ static void map_way_color(potlatch_conf_t *potlatch, way_t *way) {
       way->draw_flags |= OSM_DRAW_FLAGS_POLYGON;
       
       /* default fill color is light grey */
-      way->fill_color = (0xc0c0c0 << 8) | FILL_TRANSP;
+      way->fill_color = DEF_FILL_COLOR;
     }
   }
   
@@ -100,7 +100,7 @@ static void map_statusbar(map_t *map, map_item_t *map_item) {
       /* we don't have much space, so ignore created_by tag */
       if(strcasecmp(tag->key, "created_by") != 0) {
 	char *old = str;
-	str = g_strdup_printf("%s, %s/%s", old, tag->key, tag->value);
+	str = g_strdup_printf("%s, %s=%s", old, tag->key, tag->value);
 	g_free(old);
       }
       tag = tag->next;
@@ -116,6 +116,14 @@ static void map_outside_error(appdata_t *appdata) {
 	 _("Items must not be placed outside the working area!"));
 }
 
+static void map_item_chain_destroy(map_item_chain_t *chain) {
+  while(chain) {
+    map_item_chain_t *next = chain->next;
+    canvas_item_destroy(chain->map_item->item);
+    chain = next;
+  }
+}
+
 static void map_node_select(appdata_t *appdata, node_t *node) {
   map_t *map = appdata->map;
   map_item_t *map_item = &map->selected;
@@ -284,7 +292,7 @@ static gint map_item_destroy_event(GtkWidget *widget, gpointer data) {
 }
 
 static canvas_item_t *map_node_new(map_t *map, node_t *node, gint radius, 
-				   gint width, canvas_color_t fill, canvas_color_t border) {
+		   gint width, canvas_color_t fill, canvas_color_t border) {
 
   map_item_t *map_item = g_new0(map_item_t, 1);
   map_item->type = MAP_TYPE_NODE;
@@ -306,15 +314,42 @@ static canvas_item_t *map_node_new(map_t *map, node_t *node, gint radius,
   return map_item->item;
 }
 
+/* in the rare case that a way consists of only one node, it is */
+/* drawn as a circle. This e.g. happens when drawing a new way */
+static canvas_item_t *map_way_single_new(map_t *map, way_t *way, gint radius, 
+		   gint width, canvas_color_t fill, canvas_color_t border) {
+
+  map_item_t *map_item = g_new0(map_item_t, 1);
+  map_item->type = MAP_TYPE_WAY;
+  map_item->way = way;
+  map_item->item = canvas_circle_new(map, CANVAS_GROUP_WAYS, 
+	  way->node_chain->node->lpos.x, way->node_chain->node->lpos.y, 
+				     radius, width, fill, border);
+  
+  /* attach map_item to nodes map_item_chain */
+  map_item_chain_t **chain = &way->map_item_chain;
+  while(*chain) chain = &(*chain)->next;
+  *chain = g_new0(map_item_chain_t, 1);
+  (*chain)->map_item = map_item;
+
+  canvas_item_set_user_data(map_item->item, map_item);
+
+  canvas_item_destroy_connect(map_item->item, 
+          G_CALLBACK(map_item_destroy_event), map_item);
+
+  return map_item->item;
+}
+
 static canvas_item_t *map_way_new(map_t *map, canvas_group_t group, way_t *way, 
-				  canvas_points_t *points, gint width, canvas_color_t color,
-				  canvas_color_t fill_color) {
+		  canvas_points_t *points, gint width, canvas_color_t color,
+		  canvas_color_t fill_color) {
   map_item_t *map_item = g_new0(map_item_t, 1);
   map_item->type = MAP_TYPE_WAY;
   map_item->way = way;
 
   if(way->draw_flags & OSM_DRAW_FLAGS_POLYGON) {
-    map_item->item = canvas_polygon_new(map, group, points, width, color, fill_color);
+    map_item->item = canvas_polygon_new(map, group, points, 
+					width, color, fill_color);
   } else {
     map_item->item = canvas_polyline_new(map, group, points, width, color);
   }
@@ -346,7 +381,10 @@ static void map_way_draw(map_t *map, way_t *way) {
   /* allocate space for nodes */
   /* a way needs at least 2 points to be drawn */
   guint nodes = osm_way_number_of_nodes(way);
-  if(nodes > 1) {
+  if(nodes == 1) {
+    /* draw a single dot where this single node is */
+    map_way_single_new(map, way, S_RADIUS, 0, NODE_WAY_COLOR, 0);
+  } else {
     canvas_points_t *points = canvas_points_new(nodes);
     
     int node = 0;
@@ -358,12 +396,15 @@ static void map_way_draw(map_t *map, way_t *way) {
     
     /* draw way */
     if(way->draw_flags & OSM_DRAW_FLAGS_POLYGON) {
-      map_way_new(map, CANVAS_GROUP_POLYGONS, way, points, 2, DARK_GREY, way->fill_color);
+      map_way_new(map, CANVAS_GROUP_POLYGONS, way, points, 
+		  2, DARK_GREY, way->fill_color);
     } else {
-      map_way_new(map, CANVAS_GROUP_WAYS, way, points, WAY_WIDTH, way->color, NO_COLOR);
+      map_way_new(map, CANVAS_GROUP_WAYS, way, points, 
+		  WAY_WIDTH, way->color, NO_COLOR);
       
       if(way->draw_flags & OSM_DRAW_FLAGS_STROKE)
-	map_way_new(map, CANVAS_GROUP_WAYS_OL, way, points, STROKE_WIDTH, DARK_GREY, NO_COLOR);
+	map_way_new(map, CANVAS_GROUP_WAYS_OL, way, points, 
+		    STROKE_WIDTH, DARK_GREY, NO_COLOR);
     }
     canvas_points_free(points);
   }
@@ -522,6 +563,7 @@ static map_item_t *map_real_item_at(map_t *map, gint x, gint y) {
   /* get the item (parent) this item is the highlight of */
   map_item_t *parent = NULL;
   switch(map_item->type) {
+
   case MAP_TYPE_NODE:
     if(map_item->node->map_item_chain)
       parent = map_item->node->map_item_chain->map_item;
@@ -529,6 +571,7 @@ static map_item_t *map_real_item_at(map_t *map, gint x, gint y) {
     if(parent)
       printf("  using parent item node #%ld\n", parent->node->id);      
     break;
+
   case MAP_TYPE_WAY:
     if(map_item->way->map_item_chain)
       parent = map_item->way->map_item_chain->map_item;
@@ -536,6 +579,7 @@ static map_item_t *map_real_item_at(map_t *map, gint x, gint y) {
     if(parent)
       printf("  using parent item way #%ld\n", parent->way->id);      
     break;
+
   default:
     g_assert((map_item->type == MAP_TYPE_NODE) ||
 	     (map_item->type == MAP_TYPE_WAY)); 
@@ -561,9 +605,9 @@ static gboolean map_scroll_event(GtkWidget *widget, GdkEventScroll *event,
 
   if(event->type == GDK_SCROLL) {
     if(event->direction) 
-      map_set_zoom(appdata->map, appdata->map->zoom / 1.2);
+      map_set_zoom(appdata->map, appdata->map->zoom / ZOOM_FACTOR_WHEEL);
     else
-      map_set_zoom(appdata->map, appdata->map->zoom * 1.2);
+      map_set_zoom(appdata->map, appdata->map->zoom * ZOOM_FACTOR_WHEEL);
   }
 
   return TRUE;
@@ -783,7 +827,7 @@ static void map_touchnode_update(appdata_t *appdata, gint x, gint y) {
   /* check if we are close to one of the other nodes */
   canvas_window2world(appdata->map->canvas, x, y, &x, &y);
   node_t *node = appdata->osm->node;
-  while(node) {
+  while(!map->touchnode && node) {
     /* don't highlight the dragged node itself and don't highlight */
     /* deleted ones */
     if(((!map->pen_down.on_item ||
@@ -793,11 +837,26 @@ static void map_touchnode_update(appdata_t *appdata, gint x, gint y) {
       gint ny = y - node->lpos.y;
 	  
       if(nx*nx + ny*ny < RADIUS*RADIUS)
-	map_hl_touchnode_draw(map, node->lpos.x, node->lpos.y);
+	map_hl_touchnode_draw(map, node);
 
     }
     node = node->next;
   }
+
+  /* during way creation also nodes o0f the new way */
+  /* need to be searched */
+  if(!map->touchnode && map->action.way) {
+    node_chain_t *chain = map->action.way->node_chain;
+    while(!map->touchnode && chain) {
+      gint nx = x - chain->node->lpos.x;
+      gint ny = y - chain->node->lpos.y;
+	  
+      if(nx*nx + ny*ny < RADIUS*RADIUS)
+	map_hl_touchnode_draw(map, chain->node);
+
+      chain = chain->next;
+    }
+  }
 }
 
 static void map_button_press(map_t *map, gint x, gint y) {
@@ -926,6 +985,7 @@ static void map_button_release(map_t *map, gint x, gint y) {
       map_outside_error(map->appdata);
     else {
       node = osm_node_new(map->appdata->osm, x, y);
+      osm_node_attach(map->appdata->osm, node);
       map_node_draw(map, node);
     }
     map_action_set(map->appdata, MAP_ACTION_IDLE);
@@ -940,16 +1000,24 @@ static void map_button_release(map_t *map, gint x, gint y) {
     printf("released after WAY ADD\n");
     map_hl_cursor_clear(map);
 
-    /* convert mouse position to canvas (world) position */
-    canvas_window2world(map->canvas, x, y, &x, &y);
+    /* use the existing node if one was touched */
+    node = map_hl_touchnode_get_node(map);
+    if(node) {
+      printf("  re-using node #%ld\n", node->id);
+      map_hl_touchnode_clear(map);
+    } else {
 
-    node = NULL;
-    if(!osm_position_within_bounds(map->appdata->osm, x, y)) 
-      map_outside_error(map->appdata);
-    else {
+      /* convert mouse position to canvas (world) position */
+      canvas_window2world(map->canvas, x, y, &x, &y);
+
+      if(!osm_position_within_bounds(map->appdata->osm, x, y)) 
+	map_outside_error(map->appdata);
+      else 
+	node = osm_node_new(map->appdata->osm, x, y);
+    }
+
+    if(node) {
       g_assert(map->action.way);
-
-      node = osm_node_new(map->appdata->osm, x, y);
       osm_way_append_node(map->action.way, node);
 
       switch(osm_node_chain_length(map->action.way->node_chain)) {
@@ -966,6 +1034,13 @@ static void map_button_release(map_t *map, gint x, gint y) {
       default:
 	break;
       }
+
+      /* remove prior version of this way */
+      map_item_chain_destroy(map->action.way->map_item_chain);
+
+      /* draw current way */
+      map_way_color(map->appdata->potlatch, map->action.way);
+      map_way_draw(map, map->action.way);
     }
     break;
 
@@ -1074,7 +1149,8 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
 #else
     case '+':
 #endif
-      map_set_zoom(appdata->map, appdata->map->zoom*2);
+      map_set_zoom(appdata->map, 
+		   appdata->map->zoom*ZOOM_FACTOR_BUTTON);
       printf("zoom is now %f\n", appdata->map->zoom);
       return TRUE;
       break;
@@ -1084,7 +1160,8 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
 #else
     case '-':
 #endif
-      map_set_zoom(appdata->map, appdata->map->zoom/2);
+      map_set_zoom(appdata->map, 
+		   appdata->map->zoom/ZOOM_FACTOR_BUTTON);
       printf("zoom is now %f\n", appdata->map->zoom);
       return TRUE;
       break;
@@ -1248,8 +1325,13 @@ void map_action_set(appdata_t *appdata, map_action_t action) {
 
   case MAP_ACTION_WAY_ADD:
     printf("starting new way\n");
+    map_item_deselect(appdata);
     g_assert(!appdata->map->action.way);
-    appdata->map->action.way = g_new0(way_t, 1);
+    appdata->map->action.way = osm_way_new();
+    break;
+
+  case MAP_ACTION_NODE_ADD:
+    map_item_deselect(appdata);
     break;
 
   default:
@@ -1272,11 +1354,38 @@ void map_action_set(appdata_t *appdata, map_action_t action) {
   statusbar_set(appdata, str_state[action]);
 }
 
+
 void map_action_cancel(appdata_t *appdata) {
   map_t *map = appdata->map;
 
   switch(map->action.type) {
   case MAP_ACTION_WAY_ADD:
+    printf("  removing temporary way\n");
+    g_assert(map->action.way);
+
+    /* remove all nodes that have been created for this way */
+    /* (their way count will be 0 after removing the way) */
+    node_chain_t *chain = map->action.way->node_chain;
+    while(chain) {
+      node_chain_t *next = chain->next;
+
+      printf("    node #%ld (used by %d)\n", 
+	     chain->node->id, chain->node->ways);
+
+      chain->node->ways--;
+      if(!chain->node->ways && (chain->node->id == ID_ILLEGAL)) {
+	printf("      -> freeing temp node\n");
+	osm_node_free(chain->node);
+      }
+      g_free(chain);
+      chain = next;
+    }
+    map->action.way->node_chain = NULL;
+
+    /* remove ways visual representation */
+    map_item_chain_destroy(map->action.way->map_item_chain);
+    map->action.way->map_item_chain = NULL;
+
     osm_way_free(map->action.way);
     map->action.way = NULL;
     break;
@@ -1303,8 +1412,31 @@ void map_action_ok(appdata_t *appdata) {
 
   switch(map->action.type) {
   case MAP_ACTION_WAY_ADD:
-    printf("TODO: convert temporary way into real one\n");
-    osm_way_free(map->action.way);
+    g_assert(map->action.way);
+    
+    /* transfer all nodes that have been created for this way */
+    /* into the node chain */
+
+    /* (their way count will be 0 after removing the way) */
+    node_chain_t *chain = map->action.way->node_chain;
+    while(chain) {
+      printf("    node #%ld (used by %d)\n", 
+	     chain->node->id, chain->node->ways);
+
+      /* we can be sure that no node gets inserted twice (even if twice in */
+      /* the ways chain) because it gets assigned a non-ID_ILLEGAL id when */
+      /* being moved to the osm node chain */
+      if(chain->node->id == ID_ILLEGAL) 
+	osm_node_attach(appdata->osm, chain->node);
+
+      chain = chain->next;
+    }
+
+    /* now move the way itself into the main data structure */
+    osm_way_attach(appdata->osm, map->action.way);
+
+    map_way_select(appdata, map->action.way);
+
     map->action.way = NULL;
     break;
 
@@ -1322,14 +1454,6 @@ void map_action_ok(appdata_t *appdata) {
   map_action_set(appdata, MAP_ACTION_IDLE);
 }
 
-static void map_item_chain_destroy(map_item_chain_t *chain) {
-  while(chain) {
-    map_item_chain_t *next = chain->next;
-    canvas_item_destroy(chain->map_item->item);
-    chain = next;
-  }
-}
-
 /* called from icon "trash" */
 void map_delete_selected(appdata_t *appdata) {
   map_t *map = appdata->map;
@@ -1362,15 +1486,30 @@ void map_delete_selected(appdata_t *appdata) {
       }
 
       if(short_way) {
-	messagef(GTK_WIDGET(appdata->window), _("Node in short way"), 
-		 _("This node cannot be deleted (yet) as it's part of a way "
-		   "consisting of only two nodes."));
-	return;
+	GtkWidget *dialog = gtk_message_dialog_new(
+		   GTK_WINDOW(appdata->window),
+		   GTK_DIALOG_DESTROY_WITH_PARENT,
+		   GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
+		   _("Deleting this node will also delete one or more ways "
+		     "since they'll contain only one node afterwards. "
+		     "Do you really want this?"));
+      
+	gtk_window_set_title(GTK_WINDOW(dialog), 
+			     _("Delete node in short way(s)?"));
+  
+	/* just return here if user doesn't want this */
+	if(GTK_RESPONSE_NO == gtk_dialog_run(GTK_DIALOG(dialog))) {
+	  gtk_widget_destroy(dialog);
+	  return;
+	}
+  
+	gtk_widget_destroy(dialog);
       }
     }
 
     /* check if this node is part of a relation */
-    relation_chain_t *rel_chain = osm_node_to_relation(appdata->osm, item.node);
+    relation_chain_t *rel_chain = 
+      osm_node_to_relation(appdata->osm, item.node);
     if(rel_chain) {
       
       /* free the chain of relations */
@@ -1390,22 +1529,28 @@ void map_delete_selected(appdata_t *appdata) {
     map_item_chain_destroy(item.node->map_item_chain);
 
     /* and mark it "deleted" in the database */
-    way_chain_t *chain = osm_node_delete(appdata->osm, item.node, FALSE);
+    way_chain_t *chain = osm_node_delete(appdata->osm, item.node, FALSE, TRUE);
 
     /* redraw all affected ways */
     while(chain) {
       way_chain_t *next = chain->next;
 
       if(osm_way_number_of_nodes(chain->way) == 1) {
-	printf("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n");
-	printf("TODO: Only 1 node left on way -> delete way also\n");
-	printf("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n");
-      }
+	/* this way now only contains one node and thus isn't a valid */
+	/* way anymore. So it'll also get deleted (which in turn may */
+	/* cause other nodes to be deleted as well) */
 
-      map_item_t item;
-      item.type = MAP_TYPE_WAY;
-      item.way = chain->way;
-      map_item_redraw(appdata, &item);
+	/* remove it visually from the screen */
+	map_item_chain_destroy(chain->way->map_item_chain);
+
+	/* and mark it "deleted" in the database */
+	osm_way_delete(appdata->osm, chain->way, FALSE);
+      } else {
+	map_item_t item;
+	item.type = MAP_TYPE_WAY;
+	item.way = chain->way;
+	map_item_redraw(appdata, &item);
+      }
 
       g_free(chain);
 
@@ -1415,8 +1560,31 @@ void map_delete_selected(appdata_t *appdata) {
     break;
 
   case MAP_TYPE_WAY:
-    printf("NYI: request to delete way #%ld\n", item.way->id);
-    g_assert(0);
+    printf("request to delete way #%ld\n", item.way->id);
+
+    /* check if this way is part of a relation */
+    rel_chain = osm_way_to_relation(appdata->osm, item.way);
+    if(rel_chain) {
+      
+      /* free the chain of relations */
+      while(rel_chain) {
+	relation_chain_t *next = rel_chain->next;
+	g_free(rel_chain);
+	rel_chain = next;
+      }
+
+      messagef(GTK_WIDGET(appdata->window), _("Way in relation"), 
+       _("This way cannot be deleted (yet) as it's part of a relation"));
+
+      return;
+    }
+
+    /* remove it visually from the screen */
+    map_item_chain_destroy(item.way->map_item_chain);
+
+    /* and mark it "deleted" in the database */
+    osm_way_delete(appdata->osm, item.way, FALSE);
+
     break;
 
   default:
