@@ -27,9 +27,10 @@
 #define WAY_WIDTH        3.0
 #define STROKE_WIDTH     6.0
 #define RADIUS           4.0          // stand-alone nodes
-#define S_RADIUS         2.5          // nodes with tags (!= created_by)
+#define S_RADIUS         4.0          // nodes with tags (!= created_by)
 #define MAP_COLOR_NONE   0x0
-#define FILL_TRANSP      0x40
+#define FILL_TRANSP      0x60
+#define DEF_FILL_COLOR   CANVAS_COLOR(0xc0,0xc0,0xc0,FILL_TRANSP)
 #define NODE_COLOR       CANVAS_COLOR(0x00,0x88,0x00,0xff)
 #define NODE_WAY_COLOR   CANVAS_COLOR(0x00,0x00,0x00,0xff)
 #define HIGHLIGHT_COLOR  CANVAS_COLOR(0xff,0xff,0x00,0x80)  // stand alone nodes and ways
@@ -39,6 +40,10 @@
 #define TRACK_GPS_COLOR  CANVAS_COLOR(0x00,0x00,0x80,0xff)  // gps
 #define NO_COLOR         CANVAS_COLOR(0x00,0x00,0x00,0x00)  
 
+#define ZOOM_FACTOR_MENU   (1.5)
+#define ZOOM_FACTOR_WHEEL  (1.1)
+#define ZOOM_FACTOR_BUTTON (1.5)
+
 /* the "drag limit" is the number of pixels the mouse/pen has to */
 /* be moved so the action is actually considered dragging. This is */
 /* to prevent accidential dragging when the user only intended to click */
