@@ -230,7 +230,7 @@ tag_t *osm_parse_osm_tag(osm_t *osm, xmlDocPtr doc, xmlNode *a_node) {
 
 /* ------------------- node handling ------------------- */
 
-static void osm_node_free(node_t *node) {
+void osm_node_free(node_t *node) {
   /* there must not be anything left in this chain */
   g_assert(!node->map_item_chain);
 
@@ -333,13 +333,15 @@ void osm_node_chain_free(node_chain_t *node_chain) {
     g_assert(node_chain->node->ways);
 
     node_chain_t *next = node_chain->next;
-    node_chain->node->ways++;
+    node_chain->node->ways--;
     g_free(node_chain);
     node_chain = next;
   }
 }
 
 void osm_way_free(way_t *way) {
+  //  printf("freeing way #%ld\n", way->id);
+
   osm_node_chain_free(way->node_chain);
   osm_tags_free(way->tag);
 
@@ -1066,6 +1068,28 @@ way_t *osm_get_way_by_id(osm_t *osm, item_id_t id) {
 
 /* ---------- edit functions ------------- */
 
+item_id_t osm_new_way_id(osm_t *osm) {
+  item_id_t id = -1;
+
+  while(TRUE) {
+    gboolean found = FALSE;
+    way_t *way = osm->way;
+    while(way) {
+      if(way->id == id)
+	found = TRUE;
+
+      way = way->next;
+    }
+
+    /* no such id so far -> use it */
+    if(!found) return id;
+
+    id--;
+  }
+  g_assert(0);
+  return 0;
+}
+
 item_id_t osm_new_node_id(osm_t *osm) {
   item_id_t id = -1;
 
@@ -1089,18 +1113,14 @@ item_id_t osm_new_node_id(osm_t *osm) {
 }
 
 node_t *osm_node_new(osm_t *osm, gint x, gint y) {
-  printf("Attaching new node\n");
+  printf("Creating new node\n");
 
   node_t *node = g_new0(node_t, 1);
-  node->id = osm_new_node_id(osm);
   node->lpos.x = x; 
   node->lpos.y = y;
   node->visible = TRUE;
-  node->flags = OSM_FLAG_NEW;
   node->time = time(NULL);
 
-  /* todo: id, user ... */
-
   /* add created_by tag */
   node->tag = g_new0(tag_t, 1);
   node->tag->key = g_strdup("created_by");
@@ -1112,16 +1132,53 @@ node_t *osm_node_new(osm_t *osm, gint x, gint y) {
   printf("  new at %d %d (%f %f)\n", 
 	 node->lpos.x, node->lpos.y, node->pos.lat, node->pos.lon);
 
+  return node;
+}
+
+
+void osm_node_attach(osm_t *osm, node_t *node) {
+  printf("Attaching node\n");
+
+  node->id = osm_new_node_id(osm);
+  node->flags = OSM_FLAG_NEW;
+
   /* attach to end of node list */
   node_t **lnode = &osm->node;
   while(*lnode) lnode = &(*lnode)->next;  
   *lnode = node;
+}
 
-  return node;
+way_t *osm_way_new(void) {
+  printf("Creating new way\n");
+
+  way_t *way = g_new0(way_t, 1);
+  way->visible = TRUE;
+  way->flags = OSM_FLAG_NEW;
+  way->time = time(NULL);
+
+  /* add created_by tag */
+  way->tag = g_new0(tag_t, 1);
+  way->tag->key = g_strdup("created_by");
+  way->tag->value = g_strdup(PACKAGE " v" VERSION);
+
+  return way;
+}
+
+void osm_way_attach(osm_t *osm, way_t *way) {
+  printf("Attaching way\n");
+
+  way->id = osm_new_way_id(osm);
+  way->flags = OSM_FLAG_NEW;
+
+  /* attach to end of way list */
+  way_t **lway = &osm->way;
+  while(*lway) lway = &(*lway)->next;  
+  *lway = way;
 }
 
 /* returns pointer to chain of ways affected by this deletion */
-way_chain_t *osm_node_delete(osm_t *osm, node_t *node, gboolean permanently) {
+way_chain_t *osm_node_delete(osm_t *osm, node_t *node, gboolean permanently,
+			     gboolean affect_ways) {
   way_chain_t *way_chain = NULL, **cur_way_chain = &way_chain;
 
   /* new nodes aren't stored on the server and are just deleted permanently */
@@ -1139,9 +1196,12 @@ way_chain_t *osm_node_delete(osm_t *osm, node_t *node, gboolean permanently) {
       /* remove node from chain */
       if(node == (*chain)->node) {
 	modified = TRUE;
-	node_chain_t *next = (*chain)->next;
-	g_free(*chain);
-	*chain = next;
+	if(affect_ways) {
+	  node_chain_t *next = (*chain)->next;
+	  g_free(*chain);
+	  *chain = next;
+	} else
+	  chain = &((*chain)->next);
       } else
 	chain = &((*chain)->next);
     }
@@ -1159,10 +1219,10 @@ way_chain_t *osm_node_delete(osm_t *osm, node_t *node, gboolean permanently) {
   }
 
   if(!permanently) {
-    printf("mark as deleted\n");
+    printf("mark node #%ld as deleted\n", node->id);
     node->flags |= OSM_FLAG_DELETED;
   } else {
-    printf("permanent delete\n");
+    printf("permanently delete node #%ld\n", node->id);
 
     /* remove it from the chain */
     node_t **cnode = &osm->node;
@@ -1240,6 +1300,42 @@ relation_chain_t *osm_node_to_relation(osm_t *osm, node_t *node) {
   return rel_chain;
 }
 
+/* return all relations a way is in */
+relation_chain_t *osm_way_to_relation(osm_t *osm, way_t *way) {
+  relation_chain_t *rel_chain = NULL, **cur_rel_chain = &rel_chain;
+
+  relation_t *relation = osm->relation;
+  while(relation) {
+    gboolean is_member = FALSE;
+
+    member_t *member = relation->member;
+    while(member) {
+      switch(member->type) {
+      case WAY: {
+	/* ways can be check directly */
+	if(member->way == way)
+	  is_member = TRUE;
+      } break;
+      
+      default:
+	break;
+      }
+      member = member->next;
+    }
+
+    /* way is a member of this relation, so move it to the member chain */
+    if(is_member) {
+      *cur_rel_chain = g_new0(relation_chain_t, 1);
+      (*cur_rel_chain)->relation = relation;
+      cur_rel_chain = &((*cur_rel_chain)->next);
+    }
+
+    relation = relation->next;
+  }
+
+  return rel_chain;
+}
+
 /* return all ways a node is in */
 way_chain_t *osm_node_to_way(osm_t *osm, node_t *node) {
   way_chain_t *chain = NULL, **cur_chain = &chain;
@@ -1274,3 +1370,64 @@ gboolean osm_position_within_bounds(osm_t *osm, gint x, gint y) {
   if((y < osm->bounds->min.y) || (y > osm->bounds->max.y)) return FALSE;
   return TRUE;
 }
+
+void osm_way_delete(osm_t *osm, way_t *way, gboolean permanently) {
+
+  /* new ways aren't stored on the server and are just deleted permanently */
+  if(way->flags & OSM_FLAG_NEW) {
+    printf("About to delete NEW way -> force permanent delete\n");
+    permanently = TRUE;
+  }
+
+  /* delete all nodes that aren't in other use now */
+  node_chain_t **chain = &way->node_chain;
+  while(*chain) {
+
+    (*chain)->node->ways--;
+    printf("checking node #%ld (still used by %d)\n", 
+	   (*chain)->node->id, (*chain)->node->ways);
+
+    /* this node must only be part of this way */
+    if(!(*chain)->node->ways) {
+      /* delete this node, but don't let this actually affect the */
+      /* associated ways as the only such way is the oen we are currently */
+      /* deleting */
+      way_chain_t *way_chain = 
+	osm_node_delete(osm, (*chain)->node, FALSE, FALSE);
+      g_assert(way_chain);
+      while(way_chain) {
+	way_chain_t *way_next = way_chain->next;
+	g_assert(way_chain->way == way);
+	g_free(way_chain);
+	way_chain = way_next;
+      }
+    }
+    
+    node_chain_t *cur = (*chain);
+    *chain = cur->next;
+    g_free(cur);
+  }
+  way->node_chain = NULL;
+
+  if(!permanently) {
+    printf("mark way #%ld as deleted\n", way->id);
+    way->flags |= OSM_FLAG_DELETED;
+  } else {
+    printf("permanently delete way #%ld\n", way->id);
+
+    /* remove it from the chain */
+    way_t **cway = &osm->way;
+    int found = 0;
+
+    while(*cway) {
+      if(*cway == way) {
+	found++;
+	*cway = (*cway)->next;
+
+	osm_way_free(way);
+      } else
+	cway = &((*cway)->next);
+    }
+    g_assert(found == 1);
+  }
+}
