@@ -165,7 +165,7 @@ gboolean osm_node_in_way(way_t *way, node_t *node);
 
 void osm_node_chain_free(node_chain_t *node_chain);
 int osm_node_chain_length(node_chain_t *node_chain);
-
+void osm_node_free(node_t *node);
 
 void osm_tag_free(tag_t *tag);
 void osm_tags_free(tag_t *tag);
@@ -179,11 +179,20 @@ way_t *osm_get_way_by_id(osm_t *osm, item_id_t id);
 
 guint osm_way_number_of_nodes(way_t *way);
 relation_chain_t *osm_node_to_relation(osm_t *osm, node_t *node);
+relation_chain_t *osm_way_to_relation(osm_t *osm, way_t *way);
 way_chain_t *osm_node_to_way(osm_t *osm, node_t *node);
 
 /* ----------- edit functions ----------- */
 node_t *osm_node_new(osm_t *osm, gint x, gint y);
-way_chain_t *osm_node_delete(osm_t *osm, node_t *node, gboolean permanently);
+void osm_node_attach(osm_t *osm, node_t *node);
+way_chain_t *osm_node_delete(osm_t *osm, node_t *node, gboolean permanently,
+			     gboolean affect_ways);
+void osm_way_delete(osm_t *osm, way_t *way, gboolean permanently);
+
+way_t *osm_way_new(void);
+void osm_way_attach(osm_t *osm, way_t *way);
+
 gboolean osm_position_within_bounds(osm_t *osm, gint x, gint y);
+item_id_t osm_new_way_id(osm_t *osm);
 
 #endif /* OSM_H */
