@@ -287,139 +287,171 @@ static void appendf(struct log_s *log, const char *fmt, ...) {
     gtk_main_iteration();
 }
 
-static gboolean osm_update_item(struct log_s *log, char *xml_str, char *url,
-				char *user, item_id_t *id) {
+#define MAX_TRY 5
+
+static gboolean osm_update_item(struct log_s *log, char *xml_str, 
+			    char *url, char *user, item_id_t *id) {
+  int retry = MAX_TRY;
   char buffer[CURL_ERROR_SIZE];
 
   CURL *curl;
   CURLcode res;
 
   curl_data_t read_data;
-  read_data.ptr = xml_str;
-  read_data.len = strlen(xml_str);
-
   curl_data_t write_data;
-  write_data.ptr = NULL;
-  write_data.len = 0;
-	    
-  /* get a curl handle */
-  curl = curl_easy_init();
-  if(!curl) 
-    return FALSE;
 
-  /* we want to use our own read/write functions */
-  curl_easy_setopt(curl, CURLOPT_READFUNCTION, read_callback);
-  curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_callback);
-  
-  /* enable uploading */
-  curl_easy_setopt(curl, CURLOPT_UPLOAD, 1L);
-  
-  /* specify target URL, and note that this URL should include a file
-     name, not only a directory */
-  curl_easy_setopt(curl, CURLOPT_URL, url);
-  
-  /* now specify which file to upload */
-  curl_easy_setopt(curl, CURLOPT_READDATA, &read_data);
-  
-  /* provide the size of the upload, we specicially typecast the value
-     to curl_off_t since we must be sure to use the correct data size */
-  curl_easy_setopt(curl, CURLOPT_INFILESIZE, (long)strlen(xml_str));
+  while(retry >= 0) {
 
-  /* we pass our 'chunk' struct to the callback function */
-  curl_easy_setopt(curl, CURLOPT_WRITEDATA, &write_data);
-  
-  /* some servers don't like requests that are made without a user-agent
-     field, so we provide one */
-  curl_easy_setopt(curl, CURLOPT_USERAGENT, 
-		   PACKAGE "-libcurl/" VERSION); 
+    if(retry != MAX_TRY)
+      appendf(log, _("Retry %d/%d "), MAX_TRY-retry, MAX_TRY-1);
 
-  struct curl_slist *slist=NULL;
-  slist = curl_slist_append(slist, "Expect:");
-  curl_easy_setopt(curl, CURLOPT_HTTPHEADER, slist);
-  
-  curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, buffer);
-  
-  /* set user name and password for the authentication */
-  curl_easy_setopt(curl, CURLOPT_USERPWD, user);
-  
-  /* Now run off and do what you've been told! */
-  res = curl_easy_perform(curl);
-  
-  long response;
-  curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response);
+    /* get a curl handle */
+    curl = curl_easy_init();
+    if(!curl) {
+      appendf(log, _("CURL init error\n"));
+      return FALSE;
+    }
 
-  /* always cleanup */
-  curl_slist_free_all(slist);
-  curl_easy_cleanup(curl);
+    read_data.ptr = xml_str;
+    read_data.len = strlen(xml_str);
+    write_data.ptr = NULL;
+    write_data.len = 0;	    
 
-  printf("reply is \"%s\"\n", write_data.ptr);
+    /* we want to use our own read/write functions */
+    curl_easy_setopt(curl, CURLOPT_READFUNCTION, read_callback);
+    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_callback);
+    
+    /* enable uploading */
+    curl_easy_setopt(curl, CURLOPT_UPLOAD, 1L);
+    
+    /* specify target URL, and note that this URL should include a file
+       name, not only a directory */
+    curl_easy_setopt(curl, CURLOPT_URL, url);
+    
+    /* now specify which file to upload */
+    curl_easy_setopt(curl, CURLOPT_READDATA, &read_data);
+    
+    /* provide the size of the upload, we specicially typecast the value
+       to curl_off_t since we must be sure to use the correct data size */
+    curl_easy_setopt(curl, CURLOPT_INFILESIZE, (long)strlen(xml_str));
+    
+    /* we pass our 'chunk' struct to the callback function */
+    curl_easy_setopt(curl, CURLOPT_WRITEDATA, &write_data);
+    
+    /* some servers don't like requests that are made without a user-agent
+       field, so we provide one */
+    curl_easy_setopt(curl, CURLOPT_USERAGENT, 
+		     PACKAGE "-libcurl/" VERSION); 
+    
+    struct curl_slist *slist=NULL;
+    slist = curl_slist_append(slist, "Expect:");
+    curl_easy_setopt(curl, CURLOPT_HTTPHEADER, slist);
+    
+    curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, buffer);
+    
+    /* set user name and password for the authentication */
+    curl_easy_setopt(curl, CURLOPT_USERPWD, user);
+    
+    /* Now run off and do what you've been told! */
+    res = curl_easy_perform(curl);
+    
+    long response;
+    curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response);
+    
+    /* always cleanup */
+    curl_slist_free_all(slist);
+    curl_easy_cleanup(curl);
+    
+    printf("reply is \"%s\"\n", write_data.ptr);
+    
+    /* this will return the id on a successful create */
+    if(id && (res == 0) && (response == 200)) {
+      printf("request to parse successful reply as an id\n");
+      *id = strtoul(write_data.ptr, NULL, 10);
+    }
+    
+    g_free(write_data.ptr);
+    
+    if(res != 0) 
+      appendf(log, _("failed: %s\n"), buffer);
+    else if(response != 200)
+      appendf(log, _("failed, code: %ld %s\n"), 
+	      response, osm_http_message(response));
+    else {
+      if(!id) appendf(log, _("ok\n"));
+      else    appendf(log, _("ok: #%ld\n"), *id);
+    }
+    
+    /* don't retry unless we had an "internal server error" */
+    if(response != 500) 
+      return((res == 0)&&(response == 200));
 
-  /* this will return the id on a successful create */
-  if(id && (res == 0) && (response == 200)) {
-    printf("request to parse successful reply as an id\n");
-    *id = strtoul(write_data.ptr, NULL, 10);
+    retry--;
   }
 
-  g_free(write_data.ptr);
-  
-  if(res != 0) 
-    appendf(log, _("failed: %s\n"), buffer);
-  else if(response != 200)
-    appendf(log, _("failed, code: %ld %s\n"), 
-	    response, osm_http_message(response));
-  else {
-    if(!id) appendf(log, _("ok\n"));
-    else    appendf(log, _("ok: #%ld\n"), *id);
-  }
-
-  return((res == 0)&&(response == 200));
+  return FALSE;
 }
 
 static gboolean osm_delete_item(struct log_s *log, char *url, char *user) {
+  int retry = MAX_TRY;
   char buffer[CURL_ERROR_SIZE];
 
   CURL *curl;
   CURLcode res;
 
-  /* get a curl handle */
-  curl = curl_easy_init();
-  if(!curl) 
-    return FALSE;
+  while(retry >= 0) {
 
-  /* no read/write functions required */
-  curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, "DELETE"); 
-  
-  /* specify target URL, and note that this URL should include a file
-     name, not only a directory */
-  curl_easy_setopt(curl, CURLOPT_URL, url);
-  
-  /* some servers don't like requests that are made without a user-agent
-     field, so we provide one */
-  curl_easy_setopt(curl, CURLOPT_USERAGENT, PACKAGE "-libcurl/" VERSION); 
+    if(retry != MAX_TRY)
+      appendf(log, _("Retry %d/%d "), MAX_TRY-retry, MAX_TRY-1);
 
-  curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, buffer);
-  
-  /* set user name and password for the authentication */
-  curl_easy_setopt(curl, CURLOPT_USERPWD, user);
-  
-  /* Now run off and do what you've been told! */
-  res = curl_easy_perform(curl);
-  
-  long response;
-  curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response);
+    /* get a curl handle */
+    curl = curl_easy_init();
+    if(!curl) {
+      appendf(log, _("CURL init error\n"));
+      return FALSE;
+    }
 
-  /* always cleanup */
-  curl_easy_cleanup(curl);
-
-  if(res != 0) 
+    /* no read/write functions required */
+    curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, "DELETE"); 
+  
+    /* specify target URL, and note that this URL should include a file
+       name, not only a directory */
+    curl_easy_setopt(curl, CURLOPT_URL, url);
+    
+    /* some servers don't like requests that are made without a user-agent
+       field, so we provide one */
+    curl_easy_setopt(curl, CURLOPT_USERAGENT, PACKAGE "-libcurl/" VERSION); 
+    
+    curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, buffer);
+    
+    /* set user name and password for the authentication */
+    curl_easy_setopt(curl, CURLOPT_USERPWD, user);
+    
+    /* Now run off and do what you've been told! */
+    res = curl_easy_perform(curl);
+    
+    long response;
+    curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response);
+    
+    /* always cleanup */
+    curl_easy_cleanup(curl);
+    
+    if(res != 0) 
     appendf(log, _("failed: %s\n"), buffer);
-  else if(response != 200)
-    appendf(log, _("failed, code: %ld %s\n"), 
-	    response, osm_http_message(response));
-  else
-    appendf(log, _("ok\n"));
-
-  return((res == 0)&&(response == 200));
+    else if(response != 200)
+      appendf(log, _("failed, code: %ld %s\n"), 
+	      response, osm_http_message(response));
+    else
+      appendf(log, _("ok\n"));
+    
+    /* don't retry unless we had an "internal server error" */
+    if(response != 500) 
+      return((res == 0)&&(response == 200));
+    
+    retry--;
+  }
+  
+  return FALSE;
 }
 
 typedef struct {
@@ -450,6 +482,34 @@ static GtkWidget *table_attach_int(GtkWidget *table, int num,
   return label;
 }
 
+static void osm_delete_nodes(osm_upload_context_t *context) {
+  node_t *node = context->osm->node;
+  project_t *project = context->project;
+
+  while(node) {
+    /* make sure gui gets updated */
+    while(gtk_events_pending()) gtk_main_iteration();
+
+    if(node->flags & OSM_FLAG_DELETED) {
+      printf("deleting node on server\n");
+
+      appendf(&context->log, _("Delete node #%ld "), node->id);
+
+      char *url = g_strdup_printf("%s/node/%lu", project->server, node->id);
+      char *cred = g_strdup_printf("%s:%s", project->username, 
+				   project->password);
+
+      if(osm_delete_item(&context->log, url, cred)) {
+	node->flags &= ~(OSM_FLAG_DIRTY | OSM_FLAG_DELETED);
+	project->data_dirty = TRUE;
+      }
+      
+      g_free(cred);
+    }
+    node = node->next;
+  }
+}
+
 static void osm_upload_nodes(osm_upload_context_t *context) {
   node_t *node = context->osm->node;
   project_t *project = context->project;
@@ -464,7 +524,8 @@ static void osm_upload_nodes(osm_upload_context_t *context) {
       appendf(&context->log, _("Delete node #%ld "), node->id);
 
       char *url = g_strdup_printf("%s/node/%lu", project->server, node->id);
-      char *cred = g_strdup_printf("%s:%s", project->username, project->password);
+      char *cred = g_strdup_printf("%s:%s", project->username, 
+				   project->password);
 
       if(osm_delete_item(&context->log, url, cred)) {
 	node->flags &= ~(OSM_FLAG_DIRTY | OSM_FLAG_DELETED);
@@ -498,19 +559,43 @@ static void osm_upload_nodes(osm_upload_context_t *context) {
 	  node->flags &= ~(OSM_FLAG_DIRTY | OSM_FLAG_NEW);
 	  project->data_dirty = TRUE;
 	}
-
 	g_free(cred);
       }
-
       g_free(url);
     }
-
-    /* todo: update all ways that include this node */
-      
     node = node->next;
   }
 }
 
+static void osm_delete_ways(osm_upload_context_t *context) {
+  way_t *way = context->osm->way;
+  project_t *project = context->project;
+
+  while(way) {
+    /* make sure gui gets updated */
+    while(gtk_events_pending()) gtk_main_iteration();
+
+    if(way->flags & OSM_FLAG_DELETED) {
+      printf("deleting way on server\n");
+
+      appendf(&context->log, _("Delete way #%ld "), way->id);
+
+      char *url = g_strdup_printf("%s/way/%lu", project->server, way->id);
+      char *cred = g_strdup_printf("%s:%s", 
+				   project->username, project->password);
+
+      if(osm_delete_item(&context->log, url, cred)) {
+	way->flags &= ~(OSM_FLAG_DIRTY | OSM_FLAG_DELETED);
+	project->data_dirty = TRUE;
+      }
+      
+      g_free(cred);
+    }
+    way = way->next;
+  }
+}
+
+
 static void osm_upload_ways(osm_upload_context_t *context) {
   way_t *way = context->osm->way;
   project_t *project = context->project;
@@ -518,30 +603,47 @@ static void osm_upload_ways(osm_upload_context_t *context) {
   while(way) {
     /* make sure gui gets updated */
     while(gtk_events_pending()) gtk_main_iteration();
-    
-    if(way->flags & OSM_FLAG_DIRTY) {
-      char *url = g_strdup_printf("%s/way/%lu", project->server, way->id);
 
-      appendf(&context->log, _("Modified way #%ld "), way->id);
+    /* check if way only consists of "clean" nodes */
+    gboolean clean = TRUE;
+    node_chain_t *chain = way->node_chain;
+    while(chain) {
+      if(chain->node->flags & (OSM_FLAG_DIRTY | OSM_FLAG_NEW))
+	clean = FALSE;
+      
+      chain = chain->next;
+    }
 
+    if(!clean) 
+      appendf(&context->log, _("skip (contains dirty nodes)\n"));
+
+    if(clean && way->flags & (OSM_FLAG_DIRTY | OSM_FLAG_NEW)) {
+      char *url = NULL;
+      
+      if(way->flags & OSM_FLAG_NEW) {
+	url = g_strdup_printf("%s/way/create", project->server);
+	appendf(&context->log, _("New way "));
+      } else {
+	url = g_strdup_printf("%s/way/%lu", project->server, way->id);
+	appendf(&context->log, _("Modified way #%ld "), way->id);
+      }
+      
       /* upload this node */
       char *xml_str = osm_generate_xml_way(context->osm, way);
       if(xml_str) {
 	printf("uploading way %s from address %p\n", url, xml_str);
-
+	
 	char *cred = g_strdup_printf("%s:%s", 
-		     project->username, project->password);
-	if(osm_update_item(&context->log, xml_str, url, cred, NULL)) {
-	  way->flags &= ~OSM_FLAG_DIRTY;
+				     project->username, project->password);
+	if(osm_update_item(&context->log, xml_str, url, cred, 
+			   (way->flags & OSM_FLAG_NEW)?&(way->id):NULL)) {
+	  way->flags &= ~(OSM_FLAG_DIRTY | OSM_FLAG_NEW);
 	  project->data_dirty = TRUE;
 	}
-
 	g_free(cred);
       }
-
       g_free(url);
     }
-      
     way = way->next;
   }
 }
@@ -716,6 +818,10 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
   osm_upload_nodes(context);
   appendf(&context->log, _("Uploading ways:\n"));
   osm_upload_ways(context);
+  appendf(&context->log, _("Deleting ways:\n"));
+  osm_delete_ways(context);
+  appendf(&context->log, _("Deleting nodes:\n"));
+  osm_delete_nodes(context);
   //  appendf(&context->log, "Relations are not supported yet!\n");
 
   appendf(&context->log, _("Upload done.\n"));
@@ -743,6 +849,12 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
 
       appendf(&context->log, _("Reloading map ...\n"));
       
+      if(!diff_is_clean(appdata->osm)) {
+	appendf(&context->log, _(">>>>>>>> DIFF IS NOT CLEAN <<<<<<<<\n"));
+	appendf(&context->log, _("Something went wrong during upload,\n"));
+	appendf(&context->log, _("proceed with care!\n"));
+      }
+
       /* redraw the entire map by destroying all map items and redrawing them */
       appendf(&context->log, _("Cleaning up ...\n"));
       track_save(appdata->project, appdata->track.track);
@@ -760,6 +872,7 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
       appdata->track.track = track_restore(appdata, appdata->project);
       if(appdata->track.track)
 	map_track_draw(appdata->map, appdata->track.track);
+      wms_load(appdata);
       appendf(&context->log, _("Done!\n"));
     }
   }
